/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include"DeviceBase.h"
#include "../SensorValues/SensorValueBase.h"
#include "../util.h"
#include <IceUtil/Time.h>

namespace armarx
{
    TYPEDEF_PTRS_SHARED(SensorDevice);

    namespace SensorDeviceTags
    {
        using namespace DeviceTags;
    }

    class SensorDevice: public virtual DeviceBase
    {
    public:
        /// @brief A static const nullptr in case a const ref to a nullptr needs to be returned
        static const SensorDevicePtr NullPtr;
        /// @brief Create a SensorDevice with the given name
        SensorDevice(const std::string& name): DeviceBase {name} {}

        /// @return The SensorDevice's sensor value
        virtual const SensorValueBase* getSensorValue() const = 0;

        std::string getSensorValueType(bool withoutNamespaceSpecifier = false) const;

        /// @return The reporting frame name of this sensor (e.g. The frame some force torque values are in).
        /// This may be empty for virtual sensors e.g. execution time sensors.
        virtual std::string getReportingFrame() const;

        virtual void rtReadSensorValues(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration);
    };

    template<class SensorValueType>
    class SensorDeviceTemplate : public virtual SensorDevice
    {
        static_assert(std::is_base_of<SensorValueBase, SensorValueType>::value, "SensorValueType has to inherit SensorValueBase");
    public:
        SensorDeviceTemplate(const std::string& name): DeviceBase(name), SensorDevice(name) {}

        const SensorValueType* getSensorValue() const final override;

        SensorValueType sensorValue;
    };
}

//inline functions
namespace armarx
{
    inline std::string SensorDevice::getSensorValueType(bool withoutNamespaceSpecifier) const
    {
        return getSensorValue()->getSensorValueType(withoutNamespaceSpecifier);
    }

    inline std::string SensorDevice::getReportingFrame() const
    {
        return {};
    }

    inline void SensorDevice::rtReadSensorValues(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) {}

    template<class SensorValueType>
    inline const SensorValueType* SensorDeviceTemplate<SensorValueType>::getSensorValue() const
    {
        return &sensorValue;
    }
}

