/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::NJointKinematicUnitPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "NJointKinematicUnitPassThroughController.h"
using namespace armarx;

NJointControllerRegistration<NJointKinematicUnitPassThroughController> registrationControllerNJointKinematicUnitPassThroughController("NJointKinematicUnitPassThroughController");

NJointKinematicUnitPassThroughController::NJointKinematicUnitPassThroughController(
    NJointControllerDescriptionProviderInterfacePtr prov,
    const NJointKinematicUnitPassThroughControllerConfigPtr& cfg,
    const VirtualRobot::RobotPtr&)
{
    const SensorValueBase* sv = prov->getSensorValue(cfg->deviceName);

    std::string controlMode = cfg->controlMode;
    std::string deviceName = cfg->deviceName;

    /*if (deviceName == "ArmL5_Elb1" && controlMode == ControlModes::Velocity1DoF)
    {
        ARMARX_IMPORTANT << "overriding velocity mode for " << deviceName;
        controlMode = ControlModes::VelocityTorque;
    }*/

    ControlTargetBase* ct = prov->getControlTarget(cfg->deviceName, controlMode);
    //get sensor
    if (ct->isA<ControlTarget1DoFActuatorPosition>())
    {
        ARMARX_CHECK_EXPRESSION(sv->asA<SensorValue1DoFActuatorPosition>());
        sensor = &(sv->asA<SensorValue1DoFActuatorPosition>()->position);
        ARMARX_CHECK_EXPRESSION(ct->asA<ControlTarget1DoFActuatorPosition>());
        target = &(ct->asA<ControlTarget1DoFActuatorPosition>()->position);
        sensorToControlOnActivateFactor = 1;
    }
    else if (ct->isA<ControlTarget1DoFActuatorVelocity>())
    {
        ARMARX_CHECK_EXPRESSION(sv->asA<SensorValue1DoFActuatorVelocity>());
        sensor = &(sv->asA<SensorValue1DoFActuatorVelocity>()->velocity);
        ARMARX_CHECK_EXPRESSION(ct->asA<ControlTarget1DoFActuatorVelocity>());
        target = &(ct->asA<ControlTarget1DoFActuatorVelocity>()->velocity);
        sensorToControlOnActivateFactor = 1;
        resetZeroThreshold = 0.1f; // TODO: way to big value?!
    }
    else if (ct->isA<ControlTarget1DoFActuatorTorque>())
    {
        ARMARX_CHECK_EXPRESSION(sv->asA<SensorValue1DoFActuatorTorque>());
        sensor = &(sv->asA<SensorValue1DoFActuatorTorque>()->torque);
        ARMARX_CHECK_EXPRESSION(ct->asA<ControlTarget1DoFActuatorTorque>());
        target = &(ct->asA<ControlTarget1DoFActuatorTorque>()->torque);
        sensorToControlOnActivateFactor = 0;
    }
    else
    {
        throw InvalidArgumentException {"Unsupported control mode: " + controlMode};
    }
}
