/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::RobotUnit::RobotSynchronization
 * @author     Stefan Reither ( stef dot reither at web dot de )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/Robot.h>
#include "Devices/SensorDevice.h"

#include <string>
#include <vector>

namespace armarx
{
    class RobotUnit;
    class SensorValue1DoFActuatorPosition;

    TYPEDEF_PTRS_SHARED(RobotSynchronization);
    /**
     * @brief The RobotSynchronization class
     */
    class RobotSynchronization
    {
    public:
        RobotSynchronization(RobotUnit* robotUnit, VirtualRobot::RobotPtr& robot, const std::vector<std::string>& synchronizingJoints);
        ~RobotSynchronization() {}
        void sync();

    private:
        std::map<std::string, ConstSensorDevicePtr> jointSensorDeviceMap;
        RobotUnit* robotUnit = NULL;
        VirtualRobot::RobotPtr robot;
    };
}
