/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus (simon dot ottenhaus at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <boost/shared_ptr.hpp>

#include <Eigen/Dense>

#include "VelocityControllerHelper.h"

#include <VirtualRobot/Robot.h>

#include <RobotAPI/libraries/core/CartesianPositionController.h>
#include <RobotAPI/libraries/core/Pose.h>

namespace armarx
{
    class PositionControllerHelper;
    typedef boost::shared_ptr<PositionControllerHelper> PositionControllerHelperPtr;

    class PositionControllerHelper
    {
    public:
        PositionControllerHelper(const VirtualRobot::RobotNodePtr& tcp, const VelocityControllerHelperPtr& velocityControllerHelper, const Eigen::Matrix4f& target);
        PositionControllerHelper(const VirtualRobot::RobotNodePtr& tcp, const VelocityControllerHelperPtr& velocityControllerHelper, const std::vector<Eigen::Matrix4f>& waypoints);
        PositionControllerHelper(const VirtualRobot::RobotNodePtr& tcp, const VelocityControllerHelperPtr& velocityControllerHelper, const std::vector<PosePtr>& waypoints);

        void update();

        void setNewWaypoints(const std::vector<Eigen::Matrix4f>& waypoints);
        void setNewWaypoints(const std::vector<PosePtr>& waypoints);
        void addWaypoint(const Eigen::Matrix4f& waypoint);
        void setTarget(const Eigen::Matrix4f& target);

        float getPositionError() const;

        float getOrientationError() const;

        bool isCurrentTargetReached() const;
        bool isCurrentTargetNear() const;
        bool isFinalTargetReached() const;
        bool isFinalTargetNear() const;

        bool isLastWaypoint() const;

        const Eigen::Matrix4f& getCurrentTarget() const;

        size_t skipToClosestWaypoint(float rad2mmFactor);

        void setNullSpaceControl(bool enabled);

        CartesianPositionController posController;
        VelocityControllerHelperPtr velocityControllerHelper;

        std::vector<Eigen::Matrix4f> waypoints;
        size_t currentWaypointIndex = 0;

        float thresholdPositionReached = 0;
        float thresholdOrientationReached = 0;
        float thresholdPositionNear = 0;
        float thresholdOrientationNear = 0;
    };
}
